@@ -1,17 +1,3 @@
-"""
-What is a material?
-
-Materials define how your scene entities look like. Their colors, shininess,
-or displaying them as solid objects or wireframes, all are defined inside
-object materials. This also effects if your object will respond to light
-sources or not.
-
-There are also predefined colors in this module.
-
-* Material
-  * Shader (`payton.scene.shader`)
-
-"""
 import os
 from typing import Any, Dict, List, Optional
 
@@ -77,13 +63,10 @@ LIGHT_GRAY = [0.8, 0.8, 0.8]  # type: List[float]
 DEFAULT = "default"
 NO_VERTEX_ARRAY = -1
 NO_INDICE = -2
+EMPTY_VERTEX_ARRAY = -3
 
 
 class Material(object):
-    """
-    Material information holder.
-    """
-
     def __init__(
         self,
         color: Optional[List[float]] = None,
@@ -93,44 +76,19 @@ class Material(object):
         opacity: float = 1.0,
         **kwargs: Any,
     ):
-        """
-        Initialize Material
-
-        Color is constructed as a tuple of 3 floats. (Payton does not currently
-        support transparency at MVP.) [1.0, 1.0, 1.0] are [Red, Green, Blue]
-
-        Each element of color is a float between 0 and 1.
-        (0 - 255 respectively)
-        Also, there are predefined colors.
-
-        Display Mode has 2 modes. Solid and Wireframe. Wireframe is
-        often rendered in a faster way. Also good for debugging your
-        object.
-
-        Default variables:
-
-            {'color': [1.0, 1.0, 1.0, 1.0],
-             'display': SOLID}
-
-        Args:
-          color: Color of material
-          display: Display type of material, SOLID / WIREFRAME (Default SOLID)
-          lights: Effected by lights? (Default true)
-          texture: Texture file name
-          opacity: Opacity of the material (0 fully transparent, 1 opaque)
-        """
-
-        self.color: List[float] = [1.0, 1.0, 1.0] if color is None else color
+        self._color: List[float] = [1.0, 1.0, 1.0] if color is None else color
+        self._color_np: np.ndarray = np.array(self._color, dtype=np.float32)
         self.display: int = display
         self.lights: bool = lights
         self.texture: str = texture
         self.opacity: float = opacity
         self._image: Optional[Image] = None
         self._indices: IList = []
-        self._vao: int = -1
+        self._vao: int = NO_VERTEX_ARRAY
         self._vbos: List[int] = []
 
         self._vertex_count: int = 0
+        self._index_count: int = 0
 
         self._initialized: bool = False
         self._texture: Optional[int] = None
@@ -144,6 +102,22 @@ class Material(object):
             "indices": self._indices,
         }
 
+    @property
+    def index_count(self):
+        if self._index_count > 0:
+            return self._index_count
+        self._index_count = len(self._indices)
+        return self._index_count
+
+    @property
+    def color(self):
+        return self._color
+
+    @color.setter
+    def color(self, color: List[float]) -> None:
+        self._color = color
+        self._color_np = np.array(self._color, dtype=np.float32)
+
     @classmethod
     def from_dict(cls, d: Dict[str, Any]) -> "Material":
         res = cls()
@@ -155,11 +129,6 @@ class Material(object):
         return res
 
     def build(self) -> bool:
-        """Build material shaders
-
-        Must be called at object build stage after generating vba.
-        An active vba is required for building shader properly.
-        """
         self._initialized = True
         if os.path.isfile(self.texture):
             img = Image.open(self.texture)
@@ -175,9 +144,7 @@ class Material(object):
         glPixelStorei(GL_UNPACK_ALIGNMENT, 1)
         glBindTexture(GL_TEXTURE_2D, self._texture)
         glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
-        glTexParameterf(
-            GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR
-        )
+        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR)
         glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT)
         glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT)
 
@@ -189,45 +156,21 @@ class Material(object):
             img_data = np.fromstring(img.tobytes(), np.uint8)
             mode = GL_RGB
         glTexImage2D(
-            GL_TEXTURE_2D,
-            0,
-            mode,
-            width,
-            height,
-            0,
-            mode,
-            GL_UNSIGNED_BYTE,
-            img_data,
+            GL_TEXTURE_2D, 0, mode, width, height, 0, mode, GL_UNSIGNED_BYTE, img_data,
         )
         glGenerateMipmap(GL_TEXTURE_2D)
 
     def refresh(self) -> None:
         self._initialized = False
 
-    def render(
-        self,
-        model: np.ndarray,
-        lit: bool,
-        shader: Shader,
-        mode: Optional[int] = None,
-    ) -> None:
-        """Render material
-
-        This function must be called before rendering the actual object
-
-        Args:
-          model: Model matrix
-          shader: Shader to be used for rendering
-          lit: Is this a lit environment
-          mode: Set explicit shader mode (optional - used for vertex colors)
-        """
+    def render(self, lit: bool, shader: Shader, mode: Optional[int] = None,) -> None:
         if not self._initialized:
             self.build()
 
         _mode = Shader.LIGHT_COLOR
 
         if self.display == SOLID:
-            if lit:
+            if lit and self.lights:
                 if self._texture is not None:
                     _mode = Shader.LIGHT_TEXTURE
                 else:
@@ -246,11 +189,6 @@ class Material(object):
         glEnable(GL_BLEND)
         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
 
-        shader.set_int("material_mode", _mode)
-
-        shader.set_matrix4x4_np("model", model)
-        shader.set_float("opacity", self.opacity)
-
         if self._texture is not None:
             check = shader.get_location("tex_unit")
             if check > -1:
@@ -258,6 +196,10 @@ class Material(object):
                 glBindTexture(GL_TEXTURE_2D, self._texture)
                 shader.set_int("tex_unit", 0)
 
-        shader.set_vector3_np(
-            "object_color", np.array(self.color, dtype=np.float32)
-        )
+        if not shader._depth_shader:
+            shader.set_vector3_np("object_color", self._color_np)
+            shader.set_float("opacity", self.opacity)
+            shader.set_int("material_mode", _mode)
+            shader.set_int("lit", 1 if lit else 0)
+            if not self.lights:
+                shader.set_int("lit", 0)
